% makeprofile2.m
% Matlab 6.0 Script file for calculating the non-dimensional similarity 
% velocity functions for flow above a spinning plate. 
% Script calculates three non-dimnesional velocity profile functions:
%  vr_nd - radial velocity profile function
%  vt_nd - tangential (theta) velocity profile function
%  va_nd - axial velocity profile function
% plus the corresponding pressure profile function
%  p_nd - pressure correction profile function
%
%	A coupled finite volume algorithm is used to solve the horizontal
%   velocity profile functions, vr_nd and vt_nd, and the axial velocity profile
%	function is solved separately from the mass balance equation. The pressure 
%   correction function is calculated from the axial momentum balance.
%
%
%   The coupled solver uses a structured coupled approach. Each nodal vr,vt pair is
%   stored in a linear row in the coupled solution vector, phi. For example phi(1)
%   stores vr(1), phi(2) stores vt(1), phi(3) stores vr(2), and so one. Logical index
%   arrays are used to access the vr and vt values.
%
%   The axial advection term in the horizontal momentum equations is discrtetized
%   with a central difference scheme so the number of control volumes, n, must be
%   greater than zetal to ensure smooth converged solutions.

tolerance = 0.00001; %convergence tolerance for nonlinear iterations
countmax = 200; %maximum number of iterations/timesteps
zetal = 20.0; %non-dimensional vertical extent of domain
n = 640; %number of control volumes

dzeta = ( zetal - 0.0 ) / n; %uniform control volume spacing
dt = 5.0;	% timestep (used for relaxation)
zeta = ([1:n]*dzeta - dzeta/2.0)';	%node postions in c.v. centres

indexr = logical(rem([1:2*n],2)); %index to access radial component in coupled solution
indext = logical(ones(1,2*n) - indexr); %index to access tangential component in coupled solution

% Initial values for three velocity profile functions
vr_nd = ones(n,1);
vt_nd = ones(n,1);
va_nd = zeros(n+1,1);
dvadzeta = zeros(n,1); %axial velocity gradient
p_nd = zeros(n,1);

phi = 0.5*ones(2*n,1); %vector to hold the couple horizontal velocity fields

phi(indexr) = vr_nd;
phi(indext) = vt_nd;

zero = zeros(2,2);
converged = zeros(2*n,1); %convergence flag for each c.v.
count = 0; %iteration count
A = spalloc(2*n,2*n,24*n); %coupled coefficient matrix

while (any(~converged) & count<countmax)
   phiold = phi;
   b = zeros(2*n,1);
   count = count + 1;
   
   %wall control volume
	as = (2./dzeta+va_nd(1)/2.) * [1 0; 0 1];
	an = (1./dzeta-va_nd(2)/2.) * [1 0; 0 1];
    ap = -as -an - dzeta/dt*[1 0; 0 1];
    ap = ap -dzeta* [ phi(1) -phi(2); phi(2) phi(1)];

	A([1,2],[1,2,3,4]) = [ ap , an ];
	% Application of the wall boundary condition, u=0, v=1.
    b(1:2) = b(1:2) - as*[0 ;1.0];
    b(1:2) = b(1:2)- dzeta/dt*[1 0; 0 1]*phiold(1:2); %timestep
	b(1:2) = b(1:2) + dzeta*[0; 0]; %pressure gradient source

	% Interior control volumes
	for ii = 2 : n-1
		as = (1./dzeta+va_nd(ii)/2.) * [1 0; 0 1];
		an = (1./dzeta-va_nd(ii+1)/2.) * [1 0; 0 1];
        ap = -as -an -dzeta/dt*[1 0; 0 1];
        ap = ap - dzeta* [ phi(2*ii-1) -phi(2*ii); phi(2*ii) phi(2*ii-1)];
		A([2*ii-1,2*ii], ...
    		[2*ii-3,2*ii-2,2*ii-1,2*ii,2*ii+1,2*ii+2]) = ...
 			[as,ap,an];
 
		b(2*ii-1:2*ii) = b(2*ii-1:2*ii) - dzeta/dt*[1 0; 0 1]*phiold(2*ii-1:2*ii); %timestep
        b(2*ii-1:2*ii) = b(2*ii-1:2*ii) + dzeta*[ 0; 0]; %pressure gradient source
	end
   
   % Farfield control volume
	as = (1./dzeta+va_nd(n)/2.) * [1 0; 0 1];
	an = (2./dzeta-va_nd(n+1)/2.) * [1 0; 0 1];
    ap = -as -an - dzeta/dt*[1 0; 0 1];
    ap = ap -dzeta*[phi(2*n-1) -phi(2*n); phi(2*n) phi(2*n-1)];

	A([2*n-1,2*n],[2*n-3,2*n-2,2*n-1,2*n]) = [as,ap];

	% Application of the farfield boundary condition, u=0, v=0	
	b(2*n-1:2*n) = b(2*n-1:2*n) - an*[0; 0];
	b(2*n-1:2*n) = b(2*n-1:2*n)- dzeta/dt*[1 0; 0 1]*phiold(2*n-1:2*n); % timestep
    b(2*n-1:2*n) = b(2*n-1:2*n) + dzeta*[0; 0]; % pressure gradient source

	phi = A\b; % solution of linearized equation set
	vr_nd = phi(indexr);
    vt_nd = phi(indext);
    
   % Solution of mass balance equation for axial velocity profile function
	for ii = 2:n+1
   	va_nd(ii) = va_nd(ii-1) - 2*vr_nd(ii-1)*dzeta;
	end

    % Calculate convergence flags
	converged = abs(phi-phiold) < tolerance;

end %end of nonlinear or time loop
if count >= countmax
    error('!!!Iteration count exceeded. Solution has not converged!!!')
end

vr_nd = phi(indexr);
vt_nd = phi(indext);
    for i=1:n
        dvadzeta(i) = (va_nd(i+1)-va_nd(i))/dzeta;
    end

plot(zeta,vr_nd,zeta,vt_nd);
fprintf('The far field non-dimensional axial velocity is %g. \n',va_nd(n+1));

p_nd(n) = 0.0;
sum_n = p_nd(n) + (va_nd(n+1)+va_nd(n))^2/8.0 - dvadzeta(n);
for i=1:n-1
    vabar = (va_nd(i+1)+va_nd(i))/2.0;
    p_nd(i) = sum_n - vabar^2/2 + dvadzeta(i);
end

save profiles n dzeta zeta vr_nd vt_nd va_nd p_nd;
clear A b converged indexr indext phi phiold;